use crate::dynamic_data::{ExecutingMove, HitData, LearnedMove, Pokemon};
use crate::script_implementations::wasm::export_registry::register;
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::MoveData;
use wasmer::FunctionEnvMut;

register! {
    fn executing_move_get_user(
        env: FunctionEnvMut<WebAssemblyEnv>,
        executing_move: ExternRef<ExecutingMove>,
    ) -> ExternRef<Pokemon> {
        ExternRef::func_new(&env, executing_move.value_func(&env).unwrap().user().as_ref())
    }

    fn executing_move_get_use_move(
        env: FunctionEnvMut<WebAssemblyEnv>,
        executing_move: ExternRef<ExecutingMove>,
    ) -> ExternRef<dyn MoveData> {
        ExternRef::func_new(&env, executing_move.value_func(&env).unwrap().use_move())
    }

    fn executing_move_get_chosen_move(
        env: FunctionEnvMut<WebAssemblyEnv>,
        executing_move: ExternRef<ExecutingMove>,
    ) -> ExternRef<LearnedMove> {
        ExternRef::func_new(&env, executing_move.value_func_arc(&env).unwrap().chosen_move().as_ref())
    }

    fn executing_move_get_number_of_hits(
        env: FunctionEnvMut<WebAssemblyEnv>,
        executing_move: ExternRef<ExecutingMove>,
    ) -> u8 {
        executing_move.value_func(&env).unwrap().number_of_hits()
    }

    fn executing_move_get_hit_data(
        env: FunctionEnvMut<WebAssemblyEnv>,
        executing_move: ExternRef<ExecutingMove>,
        target: ExternRef<Pokemon>,
        hit: u8
    ) -> ExternRef<HitData> {
        ExternRef::func_new(&env, executing_move.value_func(&env).unwrap().get_hit_data(target.value_func(&env).unwrap(), hit).unwrap())
    }
}
