use crate::script_implementations::wasm::export_registry::register;
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::{MoveData, MoveLibrary};
use crate::StringKey;
use wasmer::FunctionEnvMut;

register! {
fn move_library_get_move(
    env: FunctionEnvMut<WebAssemblyEnv>,
    lib: ExternRef<dyn MoveLibrary>,
    string_key: ExternRef<StringKey>,
) -> ExternRef<dyn MoveData> {
    let lib = lib.value_func_box(&env).unwrap();
    let m = lib.get(string_key.value_func(&env).unwrap());
    if let Some(v) = m {
        ExternRef::func_new(&env, &v)
    } else {
        ExternRef::null()
    }
}

fn move_library_get_move_by_hash(env: FunctionEnvMut<WebAssemblyEnv>, lib: ExternRef<dyn MoveLibrary>, hash: u32) -> ExternRef<dyn MoveData> {
    let lib = lib.value_func_box(&env).unwrap();
    let m = lib.get_by_hash(hash);
    if let Some(v) = m {
        ExternRef::func_new(&env, &v)
    } else {
        ExternRef::null()
    }
}

fn move_data_get_name(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> ExternRef<StringKey> {
    ExternRef::func_new(&env, move_data.value_func_arc(&env).unwrap().name())
}

fn move_data_get_type(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> u8 {
    move_data.value_func_arc(&env).unwrap().move_type().into()
}
fn move_data_get_category(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> u8 {
    move_data.value_func_arc(&env).unwrap().category() as u8
}
fn move_data_get_base_power(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> u8 {
    move_data.value_func_arc(&env).unwrap().base_power()
}
fn move_data_get_accuracy(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> u8 {
    move_data.value_func_arc(&env).unwrap().accuracy()
}
fn move_data_get_base_usages(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> u8 {
    move_data.value_func_arc(&env).unwrap().base_usages()
}
fn move_data_get_target(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> u8 {
    move_data.value_func_arc(&env).unwrap().target() as u8
}
fn move_data_get_priority(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> i8 {
    move_data.value_func_arc(&env).unwrap().priority()
}
fn move_data_has_flag(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>, flag: ExternRef<StringKey>) -> u8 {
    u8::from(move_data.value_func_arc(&env).unwrap().has_flag(flag.value_func(&env).unwrap()))
}
fn move_data_has_flag_by_hash(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>, flag_hash: u32) -> u8 {
    u8::from(move_data.value_func_arc(&env).unwrap().has_flag_by_hash(flag_hash))
}
}
